--
-- Trailer Crawlers
-- This is the specialization for Crawlers from GIANTS with some tweaks to work with trailer.
--
-- @author  Stefan Geiger
-- @date  30/11/08
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
--
-- Edited by [FSM]Chefkoch @ 17/07/2015

tc = {};
	
function tc.prerequisitesPresent(specializations)
	return true;
end;

function tc:load(xmlFile)
	self.crawlers = {};
	tc.loadCrawlers(self, xmlFile, false);
end;
	
function tc:loadCrawlers(xmlFile, isInitialized)
	    local i = 0;
	    while true do
	        local key = string.format("vehicle.crawlers.crawler(%d)", i);
	        if not hasXMLProperty(xmlFile, key) then
	            break;
	        end;
	
	        local indexBase = self.components;
	        local crawler = {};
	
	        if not isInitialized then
	            local filename = getXMLString(xmlFile, key .. "#filename");
	            if filename ~= nil then
	                crawler.filename = filename;
	                crawlerIndex = Utils.getNoNil(getXMLString(xmlFile, key .. "#crawlerIndex"), "0|0");
	                linkIndex = Utils.getNoNil(getXMLString(xmlFile, key .. "#linkIndex"), "0>");
	                local i3dNode = Utils.loadSharedI3DFile(filename, self.baseDirectory, false, false, false);
	                if i3dNode ~= 0 then
	                    local loadedCrawler = Utils.indexToObject(i3dNode, crawlerIndex);
	                    local linkNode = Utils.indexToObject(self.components, linkIndex);
	                    link(linkNode, loadedCrawler);
	                    indexBase = loadedCrawler;
	                    delete(i3dNode);
	                end;
	            end;
	
	            crawler.node = Utils.indexToObject(indexBase, getXMLString(xmlFile, key.."#index"));
	            crawler.speedRefNode = Utils.getNoNil(Utils.indexToObject(indexBase, getXMLString(xmlFile, key.."#speedRefNode")), node);
	        else
	            crawler = self.crawlers[i+1];
	        end;
	
	        if crawler.node ~= nil then
	            local speedRefWheel = getXMLInt(xmlFile, key.."#speedRefWheel");
	            if speedRefWheel ~= nil and self.wheels[speedRefWheel] ~= nil then
	                crawler.speedRefWheel = self.wheels[speedRefWheel];
	            end;
	            crawler.scrollSpeed = Utils.getNoNil(getXMLFloat(xmlFile, key.."#scrollSpeed"), 1);
	            crawler.shaderParameterName = Utils.getNoNil(getXMLString(xmlFile, key.."#shaderParameterName"), "scrollPosition");
	            crawler.shaderParameterComponent = Utils.getNoNil(getXMLInt(xmlFile, key.."#shaderParameterComponent"), 1);
	            crawler.scrollPosition = 0;
	            crawler.scrollLength = Utils.getNoNil(getXMLFloat(xmlFile, key.."#scrollLength"), 1);
	
	            crawler.rotatingParts = Utils.getNoNil(crawler.rotatingParts, {});
	            local j = 0;
	            while true do
	                local keyRotation = key..string.format(".rotatingPart(%d)", j);
	                if not hasXMLProperty(xmlFile, keyRotation) then
	                    break;
	                end;
	                local rotationPart = {};
	                if not isInitialized then
	                    rotationPart.node = Utils.indexToObject(indexBase, getXMLString(xmlFile, keyRotation.."#index"));
	                else
	                    rotationPart = crawler.rotatingParts[j+1];
	                end;
	                rotationPart.speedScale = getXMLFloat(xmlFile, keyRotation.."#speedScale");
	                if rotationPart.speedScale == nil then
	                    rotationPart.speedScale = 1.0 / Utils.getNoNil(getXMLFloat(xmlFile, keyRotation.."#radius"), 1);
	                end;
	                if rotationPart.node ~= nil and not isInitialized then
	                    table.insert(crawler.rotatingParts, rotationPart);
	                end;
	                j = j + 1;
	            end;
	
	            if not isInitialized then
	                table.insert(self.crawlers, crawler);
	            end;
	        end;
	        i = i + 1;
	    end;
	end;
	
	function tc:delete()
	    for _, crawler in pairs(self.crawlers) do
	        if crawler.filename ~= nil then
	            Utils.releaseSharedI3DFile(crawler.filename, self.baseDirectory, true);
	        end;
	    end;
	end;
	
	function tc:mouseEvent(posX, posY, isDown, isUp, button)
	end;
	
	function tc:keyEvent(unicode, sym, modifier, isDown)
	end;
	
	function tc:update(dt)
	    if self:getIsActive() then
	        for i, crawler in pairs(self.crawlers) do
	            local movedDistance = 0;
	
	            if crawler.speedRefWheel ~= nil then
	
	                local newX, _, _ = getRotation(crawler.speedRefWheel.driveNode);
	                if crawler.lastRotation == nil then
	                    crawler.lastRotation = newX;
	                end;
	
	                if newX - crawler.lastRotation < -math.pi then
	                    crawler.lastRotation = crawler.lastRotation - 2*math.pi;
	                elseif newX - crawler.lastRotation > math.pi then
	                    crawler.lastRotation = crawler.lastRotation + 2*math.pi;
	                end;
	
	                movedDistance = crawler.speedRefWheel.radius * (newX-crawler.lastRotation);
	
	                if i == 1 then
	                    --print(" newX="..tostring(newX).." / "..tostring(crawler.lastRotation).." => "..tostring(movedDistance))
	                end;
	
	                crawler.lastRotation = newX;
	            else
	                local newX, newY, newZ = getWorldTranslation(crawler.speedRefNode);
	                if crawler.lastPosition == nil then
	                    crawler.lastPosition = {newX, newY, newZ};
	                end;
	                local dx, dy, dz = worldDirectionToLocal(crawler.speedRefNode, newX-crawler.lastPosition[1], newY-crawler.lastPosition[2], newZ-crawler.lastPosition[3]);
	                local movingDirection = 0;
	                if dz > 0.0001 then
	                    movingDirection = 1;
	                elseif dz < -0.0001 then
	                    movingDirection = -1;
	                end;
	                movedDistance = Utils.vector3Length(dx, dy, dz) * movingDirection;
	                crawler.lastPosition[1] = newX;
	                crawler.lastPosition[2] = newY;
	                crawler.lastPosition[3] = newZ;
	            end;
	
	            crawler.scrollPosition = (crawler.scrollPosition + movedDistance*crawler.scrollSpeed) % crawler.scrollLength;
	            if crawler.shaderParameterComponent == 1 then
	                setShaderParameter(crawler.node, crawler.shaderParameterName, crawler.scrollPosition,0,0,0, false);
	            else
	                setShaderParameter(crawler.node, crawler.shaderParameterName, 0,crawler.scrollPosition,0,0, false);
	            end;
	
	            for _, rotatingPart in pairs(crawler.rotatingParts) do
	                rotate(rotatingPart.node, rotatingPart.speedScale * movedDistance, 0, 0);
	            end;
	        end;
	    end;
	end;
	
	function tc:draw()
	end;
	
	function tc:developmentReloadFromXML(xmlFile)
	    tc.loadCrawlers(self, xmlFile, true);
	end;